#!/usr/local/bin/perl -w

# dryrun2sval2.pl v0.1
# A component of SenseTools version 0.1

# Program to convert training and test files from Senseval 1 format to Senseval 2 format

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Satanjeev Banerjee, University of Minnesota, Duluth
# bane0025@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#-----------------------------------------------------------------------------
#                              Start of program
#-----------------------------------------------------------------------------

# this program only works for version 5.6 or better!!
## require 5.6.0;

# we have to use commandline options, so use the necessary package!
use Getopt::Long;

# first check if no commandline options have been provided... in which case
# print out the usage notes!
if ( $#ARGV == -1 )
{
    &minimalUsageNotes();
    exit;
}

# now get the options!
GetOptions("dir=s", "clean_up", "numeric_answers_only", "help", "version");

# if help has been requested, print out help!
if ( defined $opt_help )
{
    $opt_help = 1;
    &showHelp();
    exit;
}

# if version has been requested, show version!
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

# if directory has been given, open it and read it, otherwise complain.
if (defined $opt_dir)
{
    if (!(-d $opt_dir))
    {
	print STDERR "$opt_dir does not look like a directory.\n";
	askHelp();
	exit;
    }

    opendir DIR, $opt_dir || die ("Couldnt open directory $opt_dir!\n");
    my @files = grep /.cor$/, readdir DIR;
    closedir DIR;

    while (@files)
    {
	push (@train_files, $opt_dir . "/" . (shift @files));
    }
}
else
{
    print STDERR "Switch --dir is a compulsory switch.\n";
    askHelp();
    exit;
}

# if we couldnt find even one file, complain and quit!
if (!(@train_files))
{
    print STDERR "No training files found in directory $opt_dir!\n";
    askHelp();
    exit;
}

# now the conversion
print "<corpus lang=\"en\">\n";

foreach $source (@train_files)
{
    # start a new instance id sequence
    my $instanceId = 1;

    # get the lexelt from the file name
    $source =~ /(\w+)\.cor/;
    $word = $1; # thats the word we are dealing with, and thats our lexelt

    print "<lexelt item=\"$word\">\n";

    # now open the source file!
    open SRC, "$source" || die "Couldnt open $source\n";
    
    my $line = <SRC>;
    while (defined $line)
    {
	# get the instance... instances are separated by blank lines
	# our definition of new line: line containing only white space
	my $instance = "";

	# keep reading until we come to a blank line or the end of the file
	while (defined $line && $line =~ /\S/)
	{
	    $instance .= " $line";
	    $line = <SRC>;
	}
	if ($instance eq "")
	{
	    if (defined $line) { $line = <SRC>; }
	    next;
	}

	# get the tags
	$instance =~ s/<tags>\"(.*)\"=\"(.*)\"<\/tags>//;
	my $tagWord = $1;
	my $answer = $2;
	
	# create answer string without the instance id... shall put that in later
	my $answerString = "";
	my @answerArray = split(/ or /, $answer);
	foreach (@answerArray)
	{
	    if (defined $opt_numeric_answers_only)
	    { 
 	        $opt_numeric_answers_only = 1;
		$_ =~ s/-.*//;
	    }
	    $answerString .= "<answer instance=\"$word\" senseid=\"$_\"/>\n";
	}

	# now to create the actual instance(s). first see if you can
	# find the whole of the tagWord string in the instance. if so
	# thats the only instance we are going to make. otherwise,
	# split the tagWord on space, and try for each separate word

	if (defined @instanceArray) { undef @instanceArray; }
	if ($instance =~ /$tagWord/)
	{
	    # cool, so we found the whole tag in one string! this
	    # means we shall have only one instance in this case.
	    $instance =~ s/$tagWord/<head>$tagWord<\/head>/;
	    push @instanceArray, $instance;
	}
	else
	{
	    # hmm, split and try
	    my @tempArray = split (/\s+/, $tagWord);
	    my $i;
	    for ($i = 0; $i <= $#tempArray; $i++)
	    {
		if ($i == 0)
		{
		    if ( $instance =~ /$tempArray[0]/ )
		    {
			$instance =~ s/$tempArray[0]/<head>$tempArray[0]<\/head>/;
			push @instanceArray, $instance;
			next; 
		    }
		}
		
		if ($instance =~ /([^>])($tempArray[$i])/)
		{
		    my $left  = $` . $1;
		    my $mid   = $2;
		    my $right = $';

		    
		    # put the <head> in, so that this string wont
		    # match future tags in the tempArray
		    $instance = $left . "<head>" . $mid . "</head>" . $right;
		    
		    # now remove previous <head> tags from the other portions of the string
		    $left =~ s/<.?head>//g;
		    $right =~ s/<.?head>//g;
		    
		    # create the new instance with just the current <head> tag
		    my $temp = $left . "<head>" . $mid . "</head>" . $right;

		    # push! 
		    push @instanceArray, $temp;
		}
	    }
	}

	# if we dont have even one instance, go to the next instance!
	if ($#instanceArray == -1) 
	{
	    if (defined $line) { $line = <SRC>; }
	    next; 
	}

	# fine so we have some instances. write the out!

	foreach $instance (@instanceArray)
	{
	    # write out the instance id
	    print "<instance id=\"$word.$instanceId\">\n";
	    
	    # now get the answer string with the correct instance id
	    my $tempAnswerString = $answerString;
	    $tempAnswerString =~ s/answer instance=\"$word\"/answer instance=\"$word.$instanceId\"/g;
	    print $tempAnswerString;
	    
	    # next the context
	    print "<context>\n";
	    
	    # do clean up if need be
	    if (defined $opt_clean_up)
	    {
		$opt_clean_up = 1;
		$instance =~ s/[^\w\s\d<>\/]//g;
		$instance = lc($instance);
		$instance =~ s/\s+/ /g;
	    }
	    
	    # finally print it, and close the context and instance thingies
	    print "$instance\n</context>\n</instance>\n";
	    
	    # increment instanceId
	    $instanceId++;
	    
	}

	$line = <SRC>;
    }
    	
    # thats it for this file. close the lexelt, and the source file

    print "</lexelt>\n";
    close SRC;
}

print "</corpus>\n";

# thats that!
    
# function to output a minimal usage note when the user has not provided any
# commandline options
sub minimalUsageNotes
{
    print "Usage: dryrun2sval2.pl [OPTIONS] --dir DIR\n";
    askHelp();
}

# function to output help messages for this program
sub showHelp
{
    print "Usage: dryrun2sval2.pl [OPTIONS] --dir DIR\n\n";

    print "Converts dry-run data of Senseval 1 to Senseval 2 format. Final xml file\n";
    print "is output to stdout.\n\n";

    print "OPTIONS (Optional Switches):\n\n";

    print "  --dir DIR                Directory containing the dry-run files.\n\n";

    print "  --clean_up               \"Cleans up\" the context. Retains only letters, numbers\n";
    print "                           and space. Removes all capitalization, converts multiple\n";
    print "                           white space to single white space, and removes new lines.\n\n";
    print "  --numeric_answers_only   Retain only numeric portion of answers. Useful in getting\n";
    print "                           rid of the non-numeric (hyphenated) portion of some answers.\n\n";
    print "  --help                   Prints this help message.\n\n";
    print "  --version                Prints the version number.\n\n";
}

# function to output the version number
sub showVersion
{
    print "dryrun2sval2.pl  -  Version 0.1\n";
    print "Copyright (C) 2000-2002, Ted Pedersen & Satanjeev Banerjee\n";
}

# function to output "ask for help" message when the user's goofed up!
sub askHelp
{
    print STDERR "Type dryrun2sval2.pl --help for help.\n";
}


